#
# Ben Tribelhorn
# 6-26-06
#

import math

def getInfo(covar):
	"""
	returns majorangle, majoraxis, minoraxis of the ellipse that would fit this covariance
	"""
	(A, B) , (B, C) = covar
	
	# find the two eigenvalues (lambdas)
	a = 1.0
	b = -(A+C)
	c = A*C-B*B
	lambda1 = (-b + math.sqrt(b*b - 4*a*c)) / (2*a)
	lambda2 = (-b - math.sqrt(b*b - 4*a*c)) / (2*a)
	if (lambda1 < lambda2):   # swap the two values
		tmp = lambda1
		lambda1 = lambda2
		lambda2 = tmp

	# now we need to find the major axis
	# the equations to solve for x and y are
	#   Ax + By = lambda x      (A-la)x = -By
	#   Bx + Cy = lambda y  or  Bx = (la-C)y
	# this is just atan2...
	#ae1, ae2 # eigenvector angles in radians
	#xe1, ye1, xe2, ye2 # unit eigenvectors 1 and 2, x and y
	ae1 = math.atan2(lambda1-A,B)
	# make sure we're between -pi/2 and pi/2
	if (ae1 < -math.pi/2.0): ae1 += math.pi
	if (ae1 > math.pi/2.0): ae1 -= math.pi
	ae2 = ae1 + math.pi/2.0  # between 0 and pi
	xe1 = math.cos(ae1)
	ye1 = math.sin(ae1)
	xe2 = math.cos(ae2)
	ye2 = math.sin(ae2)

	# how do we find the axes' lengths?
	# the eigenvalues are proportional to those lengths, but
	# we really just want to take the length of the projection
	# of each data point onto the appropriately angled axis
	# and then subtract max - min:
	max1 = 0.0
	min1 = 0.0
	max2 = 0.0
	min2 = 0.0
	#proj
	#for i in range(len):
	proj = xe1 + ye1
	if (proj > max1): max1 = proj
	if (proj < min1): min1 = proj
	proj = xe2 + ye2
	if (proj > max2): max2 = proj
	if (proj < min2): min2 = proj

	# we should be all set... ?
	majoraxisangle = ae1;
	majoraxislength = lambda1	#max1-min1;
	minoraxislength = lambda2	#max2-min2;
	
	return majoraxisangle, majoraxislength, minoraxislength

def rotate(pt, thr):
	"""
	rotate pt around the origin by thr (clockwise)
	"""
	# R = 
	# (cos, -sin)
	# (sin,  cos)
	#
	# newx = Rx
	return math.cos(thr)*pt[0] - math.sin(thr)*pt[1], math.sin(thr)*pt[0] + math.cos(thr)*pt[1]

def addPoints(p1, p2):
	return p1[0]+p2[0], p1[1]+p2[1]



